"""Представления пакета авторизации."""

from django.http import HttpRequest, HttpResponse, HttpResponsePermanentRedirect, HttpResponseRedirect, JsonResponse

def index(_: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def login_view(request: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect | HttpResponse: ...
def keycloak_callback(request: HttpRequest) -> HttpResponse | HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def logout_view(request: HttpRequest) -> HttpResponseRedirect | HttpResponsePermanentRedirect: ...
def get_new_csrftoken(request: HttpRequest) -> JsonResponse: ...
